Правила пользования утилитой aktool
===================================

aktool - консольная утилита, предназначенная для шифрования и контроля целостности данных.

СИНТАКСИС
=========

Синтаксис выполнения программы

aktool **команда** \[ опции \] \[ файл или каталог \] \...


ОПИСАНИЕ
========

Утилита предоставляет интерфейс к различным криптографическим
преобразованиям таким, как шифрование и контроль целостности данных,
выработка и проверка электронной подписи, генерация криптографических
ключей и паролей, создание и верификация сертификатов открытых ключей.

Перечисленные преобразования реализуются библиотекой *libakrypt* в
соответствии с «Принципами разработки и  модернизации шифровальных
(криптографических) средств защиты информации» для средств
класса КС3.

Каждая команда определяет отдельный класс криптографических
преобразований и функциональных возможностей. В настоящее время
доступны следующие команды:

**a, asn1parse**
: Декодирование и печать ASN.1 данных

**k, key**
: Управление ключевой информацией

**i, icode**
: Вычисление контрольной суммы или имитовставки

**s, show**
: Вывод служебной и справочной информации

**test**
: Тестирование корректности и скорости работы криптограграфических алгоритмов

Получить подробную информацию о доступных опциях и параметрах любой
команды можно с помощью вызова


    aktool команда --help


1. УПРАВЛЕНИЕ КЛЮЧЕВОЙ ИНФОРМАЦИЕЙ
===============================

## 1.1. Общее описание

Команда `key` (короткая форма `k`) позволяет вырабатывать секретные и открытые ключи
для дальнейшего использования в алгоритмах шифрования, контроля целостности,
процедурах выработки и проверки электронной подписи и т.п.

Поддерживается инфраструктура сертификатов открытых ключей, а также
генерация ключей парной связи (симметричных ключей) 
в соответствиии со схемой Блома распределения ключевой информации.

Команда `key` позволяет сохранять секретные ключи в файлы или
ключевое хранилище в контейнере, формат которого является открытым
и распространяется вместе с документацией к библиотеке *libakrypt*.
Перед сохранением секретного ключа пользователю предлагается
ввести пароль, используемый для шифрования созданного секретного ключа.
Ввод пароля можно осуществлять как в символьном, так и в шестнацатеричном формате.

Вырабатываемые открытые ключи могут сохраняться как в формате запроса на сертификат,
так и в формате собственно сертификата в соответствии с рекомендациями
Р 1323565.1.23-2018. Также команда `key` позволяет просматривать и 
изменять параметры созданных ранее ключей.

## 1.2. Опции команды key

\-a, \--algorithm \<ni\>
: Опция позволяет определить метод или алгоритм, используемый для генерации ключевой
информации. Для указания алгоритма необходимо указать его имя или идентификатор (name or identifier).
Опция необходима в некоторых схемах выработки ключей, например, при использовании схемы Блома.
По-умолчанию используется алгоритм выработки секретного ключа,
заключающийся в использовании заданного генератора случайных чисел.

\--ca-cert \<file\>
: Опция позволяет указать имя файла, содержащего сертификат открытого ключа центра сертификации.
Опция может использоваться для выработки и верификации пользовательских сертификатов.

\--ca-key \<file\>
: Опция позволяет указать имя файла, содержащего секретный ключ центра сертификации.
Опция может применяться при генерации пользовательских сертификатов из запросов на сертификат.
Данная опция является другой формой опции `--key`.

\-c, \--cert
: Опция подписывает созданный ранее запрос на сертификат открытого ключа и создает сертификат.
В качестве аргумента опции может выступать одно или несколько имен файлов,
содержащих запросы на сертификат. Также допускается использование шаблонов, например, `*.csr`. 
В случае нескольких запросов они все будут подписаны одним и тем же ключом центра сертификации.

\--curve \<ni\>
: Опция позволяет указать имя или идентификатор эллиптической кривой, которой принадлежит открытый ключ.

\--days \<count\>
: Опция позволяет определить срок действия секретного или открытого ключа.
Срок исчисляется заданным количеством дней, начиная с момента вызова утилиты.

\--field \<count\>
: Опция позволяет указать степень расширения конечного поля, в битах, используемого для реализации
криптографических алгоритмов. Допустимыми аргументами опции являются значения 256 или 512.

\--format \<value\>
: Опция позволяет указать формат хранения вырабатываемого файла с ключевой информацией.
Допустимыми аргументами опции являются значения `der`, `pem` или `certificate`.

\--id \<string\>
: Опция позволяет определить имя или идентификатор владельца ключа. В случае использования
данной опции одовременно с созданием сертификата открытого ключа или запроса на сертификат 
значение `string` определяет расширенное имя владельца.
Расширенное имя владельца должно состоять из последовательности секций вида "/имя=Значение",
где название секции должно принимать значение из множества `em`, `cn`, `su`, `ct`, `ln`, `st`, `sa`, `or` или `ou`.

\--id-hex \<string\>
: Опция позволяет указать имя владельца секретного ключа в виде шестнадцатеричной последовательности.
Данная опция не применяется при выработке сертификатов или запросов на сертификат открытого ключа.

\--inpass \<value\>
: Опция позволяет указать пароль доступа к считываемому секретному ключу в командной строке.
Использование данной опции может привести к компрометации ключевой ифнормации.

\--inpass-hex \<value\>
: Опция позволяет указать пароль доступа к считываемому секретному ключу в командной строке
в виде шестнадцатеричной последовательности.
Использование данной опции может привести к компрометации ключевой ифнормации.

\--key \<file\>
: Опция позволяет указать имя файла, содержащего считываемый секретный ключ
симметричного или асимметричного алгоритма.

\--label
: Опция позволяет присвоить секретному ключу пользовательскую метку или описание.

\-n, \--new
: Опция создает новый секретный ключ или ключевую пару для заданного криптографичеспкого алгоритма.

\--op
: Короткая форма опции `--output-public-key`.

\--outpass \<value\>
: Опция позволяет указать пароль доступа к созданному секретному ключу прямо в командной строке.
Использование данной опции может привести к компрометации ключевой ифнормации.

\--outpass-hex \<value\>
: Опция позволяет указать пароль доступа к созданному секретному ключу прямо в командной строке
в виде шестнадцатеричной последовательности.
Использование данной опции может привести к компрометации ключевой ифнормации.

\--output-public-key <\file\>
: Опция позволяет указать имя файла, в котором будет сохранен созданный сертификат или
запрос на сертификат открытого ключа. Имеет короткую форму `--op`.

\-o, \--output-secret-key \<file\>
: Опция позволяет указать имя файла, в котором будет сохранен созданный секретный ключ.

\--random \<ni\>
: Опция позволяет указать имя или идентификатор генератора псевдослучайных последовательностей,
используемого для выработки ключевой информации или электронной подписи.
По умолчанию устанавливается генератор, зависящий от операционной системы, в которой выполняется утилита.
В ОС Linux это генератор, доступ к которому обеспечивается символьным устройством `\dev\random`.

\--random-file \<file\>
: Опция позволяет в явном виде указать имя файла из которого происходит чтение псевдослучайных последовательностей.
Использование данной опции целесообразно для доступа к физическим генераторам, предоставляющим доступ к вырабатываемым 
данным посредством символьных устройств.

\-s, \--show \<file\>
: Опция позволяет вывести в простом текстовом формате
значения параметров секретного ключа (симметричного или асимметричного),
сертификата открытого ключа или запроса на сертификат. Для открытых ключей, по-возможности,
выполняется их верификация.

\--size \<value\>
: Опция позволяет указать размер ключевого множества для создаваемого секретного ключа, например, в схеме Блома.
Максимально допустимым значением в настоящее время является величина 4096.

\-t, \--target \<ni\>
: Опция позволяет указать имя криптографического алгоритма для которого предназначается создаваемый секретный ключ.
Для генерации простой незашифрованной последовательности заданного размера, не связанной ни с каким криптографическим алгоритмом,
в качестве имени алгоритма должна быть указана строка `undefined`.

\--to
: Короткая форма опции `--format`.

\-v, \--verify
: Опция выполняет верификацию открытого ключа,
который может быть задан в виде сертификата или запроса на сертификат открытого ключа.
В качестве параметров опции может быть указан один или несколько файлов, а также шаблон, например, `*.crt`.
Если указана опция `--verbose`, то утилита выводит значение открытого ключа в консоль.

Далее указаны опции, которые могут быть использованы для управления расширениями сертификатов открытых ключей.

\--authority-name
: Опция добавляет в расширение `AuthorityKeyIdentifier` расширенное имя центра сертификации.
По умолчанию данная возможность отключена.

\--ca-ext \<value\>
: Опция позволяет указать, может ли создаваемый сертификат открытого ключа
являться сертификатом центра сертификации. Допустимыми значениями опции могут являться значения `true` или `false`.

\--pathlen \<value\>
: Опция устанавливает максимальную длину цепочки сертификации. Имеет смысл только одновремено со значением `--ca-ext true`.

\--digital-signature
: Опция разрешает использование открытого ключа для проверки электронной подписи под пользовательскими данными
(в расширение `key usage` добавляется бит `Digital Signature`)

\--key-encipherment
: Опция разрешает использование открытого ключа для зашифрования ключевой информации
(в расширение `key usage` добавляется бит `Key Encipherment`)

\--data-encipherment
: Опция разрешает использование открытого ключа для зашифрования пользовательских данных
(в расширение `key usage` добавляется бит `Data Encipherment`)

\--key-agreement
: Опция разрешает использование открытого ключа в протоколах выработки ключевой информации
(в расширение `key usage` добавляется бит `Key Agreement`)

\--key-cert-sign
: Опция разрешает использование открытого ключа для проверки подписи под сертификатами открытых ключей
(соответствующий секретный ключ может быть использован для выработки подписи под сертификатами открытых ключей) 
(в расширение `key usage` добавляется бит `Certificate Sign`)

\--crl-sign
: Опция разрешает использование открытого ключа для проверки подписи под списками отозванных сертификатов
(в расширение `key usage` добавляется бит `CRL Sign`)


## 1.3. Примеры выработки ключей симметричных криптографических алгоритмов

Создать секретный ключ алгоритма блочного шифрования "Магма"
можно с помощью следующей команды.


    aktool k -nt magma -o magma.key


В процессе выполнения указанной команды пользователю будет предложено ввести пароль,
на котором будет зашифрованно значение созданного ключа. С использованием опции `--outpass`
пароль можно определить прямо в командной строке.

Аналогично, создание нового ключа выработки имитовставки для алгоритма `hmac256-streebog256`
с использованием случайных значений из символьного устройства `\dev\random`
можно выполнить с помощью следующего вызова.


    aktool k -nt hmac-streebog256 -o secret-hmac256.key \
           --label "Used for testing purpose" --random-file /dev/random


Созданный ключ будет помечен
меткой *used for testing purpose* и сохранен в файл `secret-hmac256.key`. Посмотреть 
информацию о созданном ключе можно с помощью одного из следующих вызовов.


    aktool k -s secret-hmac256.key
    aktool a secret-hmac256.key


В первом случае будет выведена информация о параметрах созданного ключа в виде простой 
таблицы следующего вида


     Тип:
        Симметричный секретный ключ
     Алгоритм:
        hmac function (hmac-streebog256, 1.2.643.7.1.1.4.1)
     Номер:
        64fd34212dad3aa2d6e3c1646237b0ab6a167833aa018aa251f6b6b9b6328a2c
     Ресурс: 65536 (key usage resource)
        от: 25 янв 2021 23:03:51 (Понедельник) MSK
        до: 25 янв 2022 23:03:51 (Вторник) MSK
     Метка:
        Used for testing purpose


Во втором случае будет выведена ASN.1 структура, 
содержащая, в частности, зашифрованное значение созданного ранее ключа.


## 1.4. Примеры выработки ключевых пар и сертификатов открытых ключей

Выработка ключевой пары должна производиться с явным указаним алгоритма электронной подписи.
Например, ключевая пара для алгоритма ГОСТ Р 34.10-2012 может быть выполнена
с помощью следующего вызова


    aktool k -nt sign256 -o aktool256.key --op aktool256_request.csr --to pem


В данном случае в процессе выполнения утилиты будет запрошена и помещена в запрос на сертификат открытого ключа
информация о владельце создаваемого ключа. Созданый запрос будет сохранен в формате `pem` (тектовый файл,
содержащий двоичную последовательность в кодировке `base64`).
Посмотреть содержимое созданных ключей можно следующим образом.


    aktool k -s aktool256.key
    aktool k -v aktool256_request.csr --verbose


Для явного определения эллиптической кривой, которой принадлежит создаваемый открытый ключ,
можно воспользоваться опции `--curve`. Для явного указания информации о владельце ключа можно воспользоваться опцией `--id`.
В следующем примере создается ключевая пара и открытый ключ помещается в самоподписаный сертификат.
Синтаксис команды отличается от предыдущего только явным указанием эллиптической кривой и аргументом опции `--to`.


    aktool k -nt sign512 --curve ec512b -o aktool512.key --op aktool512_ca.crt \
        --to certificate --id "/cn=Aktool Authority Team"


Для подписи запроса на сертификат можно воспользоваться следующей командой.


    aktool k -c aktool256_request.csr --ca-key aktool512.key  --ca-cert aktool512_ca.crt \
        --op aktool256_certificate.crt --to pem


Для верификации и просмотра созданных сертификатов можно воспользоваться следующими командами.


    aktool k -v aktool256_request.csr --verbose
    aktool k -v aktool256_certificate.crt --ca-cert aktool512_ca.crt --verbose


Добавим, что в состав исходных текстов библиотеки `libakrypt` входит скрипт `crtest.sh`,
содержащий большое количество тестов и примеров использования утилиты `aktool` для выработки и верификации
сертификатов открытых ключей.


## 1.5. Примеры выработки ключевой информации в схеме Блома

Для генерации ключей парной связи (общих симметричных ключей) в схеме Блома необходимо выработать 
мастер-ключ, представляющий собой квадратную матрицу из элементов конечного 
поля GF(2^n) для `n` равного 256 или 512 бит.
Выработать мастер-ключ можно с помощью следующего вызова, явно указав
имя алгоритма генерации ключей: `blom-master` или `blom-matrix`.


    aktool k -na blom-master --size 512 --field 256 -o master.key


В приведенном примере размер вырабатываемой матрицы составляет 512x512 элементов.
Матрица сохраняется в файл `master.key` (Следует заметить, 
что размер вырабатываемого мастер-ключа должен 
составлять 32 + `size` x (`size` + 1 ) x `field` / 16 октетов;
для приведенного примера размер будет составлять `4202528` октетов).


Для выработки ключа абонента с идентификатором `IDa` необходимо выполнить команду


    aktool k -na blom-user --key master.key -o IDa.key --id IDa


Выработанный ключ будет помещен в файл `IDa.key`.

Для выработки ключа, который будет использован абонентом с идентификатором `IDa` для связи 
с абонентом, имеющим идентификатор `IDb`, необходимо выполнить следующую команду.


    aktool k -na blom-pairwise --key IDa.key --id IDb -o pairwise-ab.plain --target undefined


Значение опции `--target` говорит о том, что выработанный ключ парной связи 
будет сохранен в файле `pairwise-ab.plain` в незашифрованом виде.
Тот же ключ может быть выработан и сохранен в формате контейнера библиотеки `libakrypt`
следующим образом.


    aktool k -na blom-pairwise --key IDa.key --id IDb -o pairwise-ab.key \ 
        --target kuznechik --to pem --label "used for testing purpose"


В этом случае, будет сформирован полноценный контейнер, содержащий мета-информацию 
о ключе парной связи. Полученный файл может быть просмотрен, например, с помощью программы `cat`
и будет иметь следующий вид.


    -----BEGIN ENCRYPTED SYMMETRIC KEY-----
    MIIBOQYJKoUDAjQBfwEBMEkGCSqFAwI0AX8CATA8BggqhQMHAQEFAjAwBggqhQMH
    AQEEAgQgAHB+xvFnuFDcr1hjTVoOzg9mrjHog+W2YMkA2nuFqiQCAgfQMIHgBgkq
    hQMCNAF/AwEwgdIGCCqFAwcBAQUCBCCvdnxc1dJHhBjMc9fJCRAheXilh54enght
    QJrU2UlDEwwYdXNlZCBmb3IgdGVzdGluZyBwdXJwb3NlMCgCAQACAyAAADAeFw0y
    MDEyMTkyMjU3NTdaFw0yMTEyMTkyMjU3NTdaMGACAQECAQAEWGA2LJdbqSbk99Wu
    vjPwoBcA/PmKb1hI8cIj9GwbbZljdGkDUvjwO1DgnfAsc7u/Fkumtvy/i/ejQEu/
    o2Mg5v5P8bJwA0SCk+XRLluRmNYVsAYS8egjFXM=
    -----END ENCRYPTED SYMMETRIC KEY-----


Добавим, что следующая последовательность команд может быть использована
абонентом с идентификатором `IDb` для получения ключа парной связи с абонентом,
связанным с идентификатором `IDa`.


    aktool k -na blom-user --key master.key -o IDb.key --id IDb
    aktool k -na blom-pairwise --key IDb.key --id IDa -o pairwise-ba.plain --target undefined 

В результате для абонента с идентификатором `IDb` будет получен ключ парной связи `pairwise-ba.plain` 
который, как легко проверить, побайтно совпадает с выработанным ранее для абонента 
с идентификатором `IDa` ключом `pairwise-ab.plain`.


    xxd -g1 pairwise-ab.plain
    00000000: e2 fd af 33 30 55 22 b3 45 fd 2f 06 2b 91 45 50  ...30U".E./.+.EP
    00000010: f5 70 a6 0d 3f 78 f7 0d 9e c4 41 b2 7b 5b f1 6b  .p..?x....A.{[.k


    xxd -g1 pairwise-ba.plain
    00000000: e2 fd af 33 30 55 22 b3 45 fd 2f 06 2b 91 45 50  ...30U".E./.+.EP
    00000010: f5 70 a6 0d 3f 78 f7 0d 9e c4 41 b2 7b 5b f1 6b  .p..?x....A.{[.k



РАЗБОР ДАННЫХ В ФОРМАТЕ ASN.1
=============================

Команда `asn1parse` позволяет декодировать и выводить в консоль данные,
закодированные в формате ASN.1 дерева. В настоящее время
поддерживается разбор чистых `der`-последовательностей, а также
der-последовательностей, закодированных в кодировке base64
(формат сертификатов `pem`).

## Опции команды asn1parse

\--convert
: Опция позволяет выполнить конвертацию ASN.1 данных между поддерживаемыми форматами,
т.е. перевести данные из `der`-формата в `pem`-формат и наоборот.
Преобразование, изменение или удаление данных из ASN.1 дерева при конвертации не производится.

\--split
: Опция позволяет разделить ASN.1 дерево на одну или несколько ветвей первого уровня.
После разделения каждая ветвь образует новое ASN.1 дерево и помещается в отдельный файл.
Опция может применяться для разделения коллекций сертификатов открытых ключей на
отдельные файлы.

\-o, \--output \<файл\>
: Опция позволяет в явном виде определить
имя файла в который будут помещены конверированные или объединенные данные.
Данную опцию имеет смысл использовать совместно с опциями `--convert` и `--join`.

\--to \<формат\>
: Опция в явном виде указывает в каком формате должны быть сохранены данные.
Доступными значениями поля формат являются: `der` и `pem`.

\--pem \<тип\>
: Опция в явном виде позволяет определить заголовок строки
в `pem`-файле, определяющий тип содержащихся далее данных.
Данную опцию имеет смысл использовать совместно с указаним опции  `--to=pem`.
При значении `--to=der` значение опции игнорируется.

## Примеры использования

Рассмотрим запрос на создание сертифката открытого ключа,
записанный в pem кодировке в файле `request_edw.pem`. Данный запрос
содержится в качестве примера в рекомендациях Р 1323565.1.023-2018, раздел А.2.2.1.


    cat request_edw.pem
    -----BEGIN CERTIFICATE-----
    MIHKMHkCAQAwEjEQMA4GA1UEAxMHRXhhbXBsZTBeMBcGCCqFAwcBAQEBMAsGCSqF
    AwcBAgEBAQNDAARAdCeV1L7ohN3yhQ/sA+o/rxhE4B2dpgtkUJOlXibfw5l49ZbP
    TU0MbPHRiUPZRJPRa57AoW1RLS4SfMRpGmMY4qAAMAoGCCqFAwcBAQMCA0EAG9wq
    Exdnm2YjL2PqFv98ZMyqua2FX8bhgJFmHbedSBIdDh2lvjR8bxtSVseurCAK1krH
    em9bOg4Jcxjnrm7naQ==
    -----END CERTIFICATE-----

Декодирование данного файла сможет быть выполнено с помощью следующего вызова.


    aktool a request_edw.pem


В результате декодирования в консоль должна быть выведена следующая
иерархическая структура.


    ┌SEQUENCE┐
         ├SEQUENCE┐
         │        ├INTEGER 0x0
         │        ├SEQUENCE┐
         │        │        └SET┐
         │        │            └SEQUENCE┐
         │        │                     ├OBJECT IDENTIFIER 2.5.4.3
         │        │                     └PRINTABLE STRING Example
         │        ├SEQUENCE┐
         │        │        ├SEQUENCE┐
         │        │        │        ├OBJECT IDENTIFIER 1.2.643.7.1.1.1.1
         │        │        │        └SEQUENCE┐
         │        │        │                 └OBJECT IDENTIFIER 1.2.643.7.1.2.1.1.1
         │        │        └BIT STRING
         │        │           04 40 74 27 95 D4 BE E8 84 DD F2 85 0F EC 03 EA
         │        │           3F AF 18 44 E0 1D 9D A6 0B 64 50 93 A5 5E 26 DF
         │        │           C3 99 78 F5 96 CF 4D 4D 0C 6C F1 D1 89 43 D9 44
         │        │           93 D1 6B 9E C0 A1 6D 51 2D 2E 12 7C C4 69 1A 63
         │        │           18 E2
         │        └[0]┐
         │             (null)
         ├SEQUENCE┐
         │        └OBJECT IDENTIFIER 1.2.643.7.1.1.3.2
         └BIT STRING
            1B DC 2A 13 17 67 9B 66 23 2F 63 EA 16 FF 7C 64
            CC AA B9 AD 85 5F C6 E1 80 91 66 1D B7 9D 48 12
            1D 0E 1D A5 BE 34 7C 6F 1B 52 56 C7 AE AC 20 0A
            D6 4A C7 7A 6F 5B 3A 0E 09 73 18 E7 AE 6E E7 69


Следующий вызов демонстрирует
пример конвертирования ASN.1 дерева, сохраненного в виде
`der`-последовательности, в формат `pem`.


    aktool a --convert request_edw.p10 --to pem --pem request -o request_edw.pem


Следующий вызов иллюстрирует, как можно разделить коллекцию сертификатов
`ca-bundle.crt` на отдельные файлы.


    aktool a --split ca-bundle.crt --to pem --pem certificate



ВЫВОД СЛУЖЕБНОЙ И ВСПОМОГАТЕЛЬНОЙ ИНФОРМАЦИИ
============================================

Команда `show` позволяет получить информацию о доступных
преобразованиях, параметрах криптографических ключей, значениях технических и
криптографических характеристик, а также параметрах библиотеки по-умолчанию. В
настоящий момент доступны следующие опции.

## Опции команды show

\--curve
: Опция позволяет вывести в консоль параметры эллиптической кривой с заданным именем
или идентификатором.

\--engines
: Опция позволяет вывести список всех типов криптографических
механизмов, которые могут быть использованы в приложениях библиотеки
*libakrypt*. К таким типам, например, относятся блочные шифры или
алгоритмы выработки имитовставки.

\--oid \<eni\>
: Опция принимает в качестве параметра произвольную строку символов
`eni` (engine, name or identifier) и выводит все доступные
криптографические механизмы, в именах или идентификаторах которых
содержится указанная строка.

\--oids
: Опция позволяет вывести список всех доступных криптографических
механизмов. Каждый механизм идентифицируется свои именем, которое
используется для его указания пользователем и/или
последовательностью чисел, разделенных точками. Последняя
последовательность называется Object IDentifier (OID) и используется
при автоматизированной обработке данных.

\--options
: Опция выводит перечень всех криптографических параметров библиотеки,
а также их значений. К указанным параметрам относятся, например,
число блоков информации, которые могут быть зашифрованы на одном
ключе, или число итераций криптографической функции PBKDF2,
используемой для развертки ключа из пароля. Значения параметров
содержатся в файле `libakrypt.conf`, расположенном либо в 

\--modes
: Опция позволяет вывести список всех режимов криптографических
преобразований, которыми могут быть охарактеризованы
криптографические преобразования. При этом одному типу криптографического
механизма может соответствовать несколько режимов. Например, функция
хеширования имеет режим `algorithm`, а ее параметры `kbox params`.

\--without-caption
: Опция запрещает печать заголовка, расшифровывающего названия
выводимых параметров и их значений.


## Примеры использования

Наиболее употребимым вариантом вызова команды `show` является вызов


    aktool s --oids

В результате данного вызова будет выведена информация
о всех доступных библиотеке *libakrypt* криптографических преобразованиях.
Например, среди прочих, будет выведена  информация об алгоритме
блочного шифрования Кузнечик, см. ГОСТ Р 34.12-2015.


     N  oid                 name(s)          engine            mode
    --------------------------------------------------------------------------
     9  1.2.643.7.1.1.5.2   kuznechik        block cipher      algorithm
                            kuznyechik
                            grasshopper

В данной таблице указаны

 - порядковый номер криптографического механизма,
 - идентификатор (последовательность цифр, разделенных точками),
 - перечень имен, с помощью которых пользователь может именовать
   криптографический механизм,
 - тип криптографического механизма,
 - режим использования.

Выборочный поиск можно осуществить с помощью опции `--oid` (без `s` на конце). Например,
следующий вызов позволяет найти все идентификаторы, имена которых
регламентированы Техническим комитетом *Криптографическая защита информации* (ТК26).


    aktool s --oid tc26


Получить параметры эллиптической кривой можно с помощью следующего вызова.


    aktool s --curve axel-n0


После чего, в консоль будет выведена следующая информация.


    curve: id-axel-gost-3410-2012-256-paramSetN0
           axel-n0
    oid:   1.2.643.2.52.1.12.1.1
    forms:
      short Weierstrass form:   y^2 = x^3 + ax + b (mod p)
    
    parameters:
      a =  0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffffffd2158
      b =  0x42dfde56dd26bb76eba94ce9565e562bed1fb994675632a264afef327aa4e5ff
      p =  0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffffffd215b
      q =  0xffffffffffffffffffffffffffffffff2c1b759991830c6b5dcc785b195c4edb
      c =  0x01 [cofactor]
      
    point:
      px =  0x0000000000000000000000000000000000000000000000000000000000000002
      py =  0x011e47b6e40dc7f783b9f4fc84d085884b9b88ca9ec7da8c5567c9d87f68a17f

Перечень всех известных библиотеке идентификаторов эллиптических кривых
может быть получен с помощью следующего вызова.


    aktool s --oid curve



ОПЦИИ, ОБЩИЕ ДЛЯ ВСЕХ КОМАНД
============================

\--audit \<level\>
: Опция позволяет задать уровень вывода сообщений работы программы 
(в виде целого неотрицательного числа). На уровне 0 выводятся только сообщения об ошибках.
Уровень 1 определяет минимальный набор выводимых сообщений сообщений, 
в то время как уровень 2 позволяет вывести дополнительные сведения о 
выполняющихся криптографических преобразованиях.

\--audit-file \<file\>
: Опция позволяет определить файл, в который будут помещаться сообщения о процессе работы программы 
и ошибках её выполнения. В случае, если аргумент опции принимает значение `stderr`,
то используется системный канал вывода ошибок.

\--dont-use-colors
: По умолчанию для акцентирования внимания пользователя утилита использует цветовое выделение при 
выводе некоторых данных. К таким данным относятся коды ошибок, сообщения об автоматическом 
декодировании ASN.1 деревьев и т.п. Данная опция запрещает цветовое выделения.

\--hex-input
: Опция указывает, что все вводимые с консоли пароли и идентификаторы должны интерпретироваться
как последовательности шестнадцатеричных символов.

\--openssl-style
: Опция позволяет использовать нестандартизированные форматы представления данных 
для некоторых криптографических алгоритмов, аналогично тому, 
как они реализованы в библиотеке `openssl`.

\--verbose
: Опция позволяет вывести в консоль дополнительные сведения о выполняемых операциях. 
Также опция расширяет перечень сообщений, выводимых с помощью установленной системы аудита.


## Примеры использования

Следующий вызов содержит
пример перенаправления вывода утилиты `aktool` с запрещением использования цвета
на вход программы просмотра `less` (запрет использования цвета убирает управляющие команды,
которые не могут быть обработаны утилитой просмотра).


    aktool a 0408435EB90E5C8796A160E69E4BFAC453435D1D.cer --dont-use-colors | less



СТАНДАРТЫ
=========

## Национальные стандарты Российской Федерации
 - ГОСТ Р 34.10-2012 Процессы формирования и проверки электронной подписи,
 - ГОСТ Р 34.11-2012 Функция хэширования,
 - ГОСТ Р 34.12-2015 Блочные шифры,
 - ГОСТ Р 34.13-2015 Режимы работы блочных шифров.

## Рекомендации по стандартизации Росстандарта России
 - Р 50.1.111-2016 Парольная защита ключевой информации,
 - Р 50.1.113-2016 Криптографические алгоритмы, сопутствующие применению алгоритмов
   электронной цифровой подписи и функции хэширования,
 - Р 1323565.1.012-2017 Принципы разработки и модернизации шифровальных (криптографических) средств защиты информации,
 - Р 1323565.1.017-2018 Криптографические алгоритмы, сопутствующие применению алгоритмов блочного шифрования.
 - Р 1323565.1.023-2018 Использование алгоритмов ГОСТ Р 34.10-2012, ГОСТ Р 34.11-2012 в сертификате,
   списке аннулированных сертификатов (CRL) и запросе на сертификат PKCS #10 инфраструктуры открытых ключей X.509,
 - Р 1323565.1.024–2019 Параметры эллиптических кривых для криптографических алгоритмов и протоколов.
 - Р 1323565.1.026–2019 Режимы работы блочных шифров, реализующие аутентифицированное шифрование.
 
ИНФОРМАЦИЯ О ПРОЕКТЕ
====================

Интернет-сайт проекта http://libakrypt.org

Исходные коды утилиты и библиотеки:
https://github.com/axelkenzo/libakrypt-0.x

АВТОРЫ
======

Axel Kenzo & The Company Of Belles Lettres (с) 2014 - 2021.
